
## ECONOMETRIC METHODS FOR FORECASTING
#
#  VALUE-AT-RISK WITH GARCH(1,1)
#
#  Description:
#  This code snippet shows how to calculate and plot
#  the value-at-risk (VaR) with a given GARCH filter.
#  In this example, the parameters of the GARCH are
#  already fixed at their point estimates.
#
#  Francisco Blasques 2019

import numpy as np
from scipy import stats
from mat4py import loadmat
import statsmodels
import pandas as pd
from scipy.stats import norm
import matplotlib.pyplot as plt


## 1. Load data

x = loadmat('stock_returns.mat')
dfX =  pd.DataFrame({'x': np.array(x['x'][:])[:,0]})
x = dfX['x'].values
T = len(x)
x = x.reshape(T,1)

## 2. Parameter Values



## 3. Filter Volatilites
sig = np.zeros((T+1,1))
sig[0,:] = np.var(x)

for t in range(1, T):
    sig[t+1,:] = 0.11 + 0.08*x[t,:]**2 + 0.91*sig[t,:]



## 4. Calculate VaR and ES
VaR10  = np.zeros((T+1,1))
VaR05  = np.zeros((T+1,1))
VaR01  = np.zeros((T+1,1))
for t in range(1,T):
    VaR10[t,:] = norm.ppf(0.1, loc=0, scale=np.sqrt(sig[t,:]))
    VaR05[t,:] = norm.ppf(0.05, loc=0, scale=np.sqrt(sig[t,:]))
    VaR01[t,:] = norm.ppf(0.01, loc=0, scale=np.sqrt(sig[t,:]))




# Creates two subplots and unpacks the output array immediately
f, (ax1, ax2) = plt.subplots(2, 1, sharey=True)
ax1.plot(x)
ax1.plot(sig[0:-1]**(1/2))
ax1.set_title('data')
ax2.plot(VaR10[:-1])
ax2.plot(VaR05[:-1])
ax2.plot(VaR01[:-1])
ax2.set_title('VaR')
plt.show()
